/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui;

import eu.svjatoslav.sixth.e3d.geometry.Box;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.humaninput.MouseInteractionController;
import eu.svjatoslav.sixth.e3d.gui.humaninput.UserInputHandler;
import eu.svjatoslav.sixth.e3d.gui.textEditorComponent.KeyboardHelper;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.wireframe.WireframeBox;

import java.awt.event.KeyEvent;

public class GuiComponent extends AbstractCompositeShape implements
        UserInputHandler, MouseInteractionController {

    private static final String GROUP_GUI_FOCUS = "gui.focus";
    public final ViewPanel viewPanel;
    Box containingBox = new Box();
    private WireframeBox borders = null;

    private boolean borderShown = false;

    public GuiComponent(final Transform transform,
                        final ViewPanel viewPanel, final Point3D size) {
        super(transform);
        this.viewPanel = viewPanel;
        setDimensions(size);
    }

    @Override
    public boolean beforeRender(final ViewPanel viewPanel,
                                final int millisecondsSinceLastFrame) {
        return false;
    }

    private WireframeBox createBorder() {
        final LineAppearance appearance = new LineAppearance(10,
                new eu.svjatoslav.sixth.e3d.renderer.raster.Color(255, 0, 0, 100));

        final double borderSize = 10;

        final Box borderArea = containingBox.clone().addBorder(borderSize);

        return new WireframeBox(borderArea, appearance);
    }

    @Override
    public boolean focusLost(final ViewPanel viewPanel) {
        hideBorder();
        return true;
    }

    @Override
    public boolean focusReceived(final ViewPanel viewPanel) {
        showBorder();
        return true;
    }

    public WireframeBox getBorders() {
        if (borders == null)
            borders = createBorder();
        return borders;
    }

    public int getDepth() {
        return (int) containingBox.getDepth();
    }

    public int getHeight() {
        return (int) containingBox.getHeight();
    }

    public int getWidth() {
        return (int) containingBox.getWidth();
    }

    public void hideBorder() {
        if (!borderShown)
            return;
        borderShown = false;
        removeGroup(GROUP_GUI_FOCUS);
    }

    @Override
    public boolean keyPressed(final KeyEvent event, final ViewPanel viewPanel) {
        if (event.getKeyChar() == KeyboardHelper.ESC)
            viewPanel.getKeyboardFocusStack().popFocusOwner();
        return true;
    }

    @Override
    public boolean keyReleased(final KeyEvent event, final ViewPanel viewPanel) {
        return false;
    }

    @Override
    public void mouseClicked() {
        viewPanel.getKeyboardFocusStack().pushFocusOwner(this);
    }

    @Override
    public void mouseEntered() {
    }

    @Override
    public void mouseExited() {
    }

    private void setDimensions(final Point3D size) {
        containingBox.setSizeCentered(size);
    }

    private void showBorder() {
        if (borderShown)
            return;
        borderShown = true;
        addShape(getBorders(), GROUP_GUI_FOCUS);
    }

}
